"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.throttle = void 0;

// This is pretty much taken from underscore.js
// Removed context handling as we don't need it, and only leading is optional,
// trailing is not
const throttle = (func, wait, leading = false) => {
  let timeout, lastArgs;
  let previousCall = 0;

  const later = function () {
    previousCall = !leading ? 0 : Date.now();
    timeout = null;
    func(...lastArgs); // Check here prevents issues if we call the throttled fn recursively

    if (!timeout) lastArgs = null;
  };

  const throttled = (...args) => {
    const now = Date.now(); // On the very first call this will fake that we just called it to skip the leading call

    if (!previousCall && !leading) previousCall = now; // How long till we should call the function again?

    const remaining = wait - (now - previousCall); // Store these away for when we actually call it

    lastArgs = args; // If time went backwards remaining may be negative, in which case just call it.
    // Otherwise, call it if we have waited long enough.

    if (remaining <= 0 || remaining > wait) {
      // Clear the trailing edge timeout since we are about to just call the function
      if (timeout) {
        clearTimeout(timeout);
        timeout = null;
      }

      previousCall = now;
      func(...lastArgs); // Check here prevents issues if we call the debounce fn recursively

      if (!timeout) lastArgs = null;
    } else if (!timeout) {
      // If we didn't just call the function because we have waited long enough,
      // then set a timeout for the trailing edge call if we haven't done that already
      timeout = setTimeout(later, remaining);
    }
  };

  return throttled;
};

exports.throttle = throttle;